/* mbed Microcontroller Library
* Copyright (c) 2006-2015 ARM Limited
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <mbed.h>
#include "ble/BLE.h"
#include "ble/DiscoveredCharacteristic.h"
#include "ble/DiscoveredService.h"
#include "PinNames.h"
#include "mbed_events.h"
#include "UARTService.h"
#include "RTT/SEGGER_RTT.h"
#include "FaultHandler.h"
Serial device(P0_29, P0_30, 9600); //tx, rx, Baud
UARTService *nus;

#define BLINKY_ENABLED            1   //define this as 0 to disable connection status-led (led1)


DigitalOut alivenessLED(LED1);
static EventQueue eventQueue(/* event count */ 16 * EVENTS_EVENT_SIZE);

void periodicCallback(void) {
    nus->writeString("NORDIC DEMO");
    nus->flush();
    alivenessLED = !alivenessLED; /* Do blinky on LED1 while we're waiting for BLE events */
   
}

void logInfo(const char *fmt, ...)
{
    char buf[512]; //no initializing for performance
    va_list args;
    va_start(args, fmt);
    vsprintf(buf, fmt, args);
    SEGGER_RTT_WriteString(0, buf);
    SEGGER_RTT_Write(0,"\n",1); //append newline
    va_end(args);
}


void ble_init_Advertiser()
{
    #define DEVICE_NAME "BLE-DEVICE"
    BLE &ble = BLE::Instance();
    nus = new UARTService(ble);
    // setup advertising
    ble.gap().accumulateAdvertisingPayload(
        GapAdvertisingData::BREDR_NOT_SUPPORTED |
        GapAdvertisingData::LE_GENERAL_DISCOVERABLE);
    ble.gap().accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LOCAL_NAME, (uint8_t *)DEVICE_NAME,strlen(DEVICE_NAME));
        
    ble.gap().accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LIST_128BIT_SERVICE_IDS, (const uint8_t *)UARTServiceUUID_reversed, sizeof(UARTServiceUUID_reversed));

    ble.gap().setAdvertisingType(
        GapAdvertisingParams::ADV_CONNECTABLE_UNDIRECTED);

    ble.gap().setAdvertisingTimeout(100);
    ble.gap().setAdvertisingInterval(100); /* 100ms */
    ble.gap().setTxPower(4);               //max txPower = -40, min txPower = 4
}


void bleInitComplete(BLE::InitializationCompleteCallbackContext *params)
{
   BLE&        ble   = params->ble;
   ble_error_t error = params->error;

   if (error != BLE_ERROR_NONE) {
       /* In case of error, forward the error handling to onBleInitError */
       return;
   }

   /* Ensure that it is the default instance of BLE */
   if (ble.getInstanceID() != BLE::DEFAULT_INSTANCE) {
       return;
   }

   ble_init_Advertiser();

}

void scheduleBleEventsProcessing(BLE::OnEventsToProcessCallbackContext* context) {
   BLE &ble = BLE::Instance();
   eventQueue.call(Callback<void()>(&ble, &BLE::processEvents));
}


int main()
{
   SEGGER_RTT_Init();
   eventQueue.call_every(500, periodicCallback);   //Calls an event on the queue periodically

   BLE &ble = BLE::Instance();
   ble.onEventsToProcess(scheduleBleEventsProcessing);
   ble.init(bleInitComplete);
   ble.gap().startAdvertising();
   eventQueue.dispatch_forever();

   return 0;
}
